using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class SophieNetworkScript : MonoBehaviour {
	
	
	private bool isServer = false;
	private string gameName = "TestSession";
	
	
	private bool connected = false;
	
	public GameObject fpsEntityPrefab;
	
	private List<FPSEntity> fpsEntities = new List<FPSEntity>();
	
	public NetworkViewID netviewID;
	
	public string playerName = "DefaultPlayerName";
	
	
	private bool customising = false;
	private Color colA = Color.red;
	private Color colB = Color.green;
	private Color colC = Color.cyan;
	
	public void SendPlayer(NetworkViewID viewID, Vector3 pos, Vector3 ang, bool crouch, Vector3 moveVec){
		networkView.RPC("SendPlayerRPC", RPCMode.Others, viewID, pos, ang, crouch, moveVec);
	}
	
	[RPC]
	void SendPlayerRPC(NetworkViewID viewID, Vector3 pos, Vector3 ang, bool crouch, Vector3 moveVec){
		for (int i=0; i<fpsEntities.Count; i++){
			if (viewID == fpsEntities[i].viewID){
				fpsEntities[i].UpdatePlayer(pos, ang, crouch, moveVec);
			}
		}
	}
	
	void InstantiateFPSEntity(bool isLocal, NetworkViewID anID, Color cA, Color cB, Color cC){
		GameObject newEntity = (GameObject)GameObject.Instantiate(fpsEntityPrefab);
		FPSEntity entity = newEntity.GetComponent<FPSEntity>();
		entity.theNetwork = this;
		
		entity.colA = cA;
		entity.colB = cB;
		entity.colC = cC;
		
		if (isLocal){
			entity.viewID = netviewID;
			entity.isLocal = true;
		}else{
			entity.viewID = anID;
			entity.isLocal = false;
		}
		fpsEntities.Add(entity);
	}
	
	void OnGUI(){
		if (!connected){
			
			
			
			
			//set up server
			gameName = GUILayout.TextField(gameName);
			if (GUILayout.Button("Init Server")){
				bool useNat = !Network.HavePublicAddress();
				Debug.Log("Initialising server, has public address: " + Network.HavePublicAddress().ToString());
				Network.InitializeServer(32,25000, useNat);
				
			}
			
			GUILayout.Label("~~~~~~~~~");
			//connect to a server
			
			if (GUILayout.Button("Request Host List")){
				MasterServer.RequestHostList("SophiesTestGame");
			}
			HostData[] hostData = MasterServer.PollHostList();
			
			for (int i=0; i<hostData.Length; i++){
				GUILayout.BeginHorizontal();
				GUILayout.Label(" - " + hostData[i].gameName);
				if (GUILayout.Button("Connect")){
					Network.Connect(hostData[i]);
				}
				GUILayout.EndHorizontal();
			}
			
			
			
			GUILayout.Label("~~~~~~~~~");
			
			string customisingString = "\\/ Customise character \\/";
			if (customising) customisingString = "/\\ Stop customising /\\";
			
			if (GUILayout.Button(customisingString)) customising = !customising;
			
			if (customising){
			playerName = GUILayout.TextField(playerName);
			
			GUILayout.Label("Colour A:");
			colA.r = GUILayout.HorizontalSlider(colA.r,0f,1f);
			colA.g = GUILayout.HorizontalSlider(colA.g,0f,1f);
			colA.b = GUILayout.HorizontalSlider(colA.b,0f,1f);
			
			GUILayout.Label("Colour B:");
			colB.r = GUILayout.HorizontalSlider(colB.r,0f,1f);
			colB.g = GUILayout.HorizontalSlider(colB.g,0f,1f);
			colB.b = GUILayout.HorizontalSlider(colB.b,0f,1f);
			
			GUILayout.Label("Colour C:");
			colC.r = GUILayout.HorizontalSlider(colC.r,0f,1f);
			colC.g = GUILayout.HorizontalSlider(colC.g,0f,1f);
			colC.b = GUILayout.HorizontalSlider(colC.b,0f,1f);
			}
		
		}
		
			
		
		if (connected){
			if (GUILayout.Button("Disconnect Server")){
				
				
				if (!isServer){
					networkView.RPC("PlayerLeave", RPCMode.OthersBuffered, netviewID);
				}else{
					networkView.RPC("ServerLeave", RPCMode.OthersBuffered);
				}
				
				
				Network.Disconnect();
				if (isServer) MasterServer.UnregisterHost();
				connected = false;
				isServer = false;
				
				Camera.main.transform.parent = null;
				for (int i=0; i<fpsEntities.Count; i++){
					Destroy(fpsEntities[i].gameObject);
				}
				fpsEntities = new List<FPSEntity>();
			}
			
			if (isServer){
				GUILayout.Label("You are Host");
			}else{
				GUILayout.Label("You are Client");
			}
			
		}
		
		
		
		
	}
	
	[RPC]
	void NewPlayer(NetworkViewID viewID, string name, Vector3 cA, Vector3 cB, Vector3 cC){
		InstantiateFPSEntity(false, viewID, VecToCol(cA), VecToCol(cB), VecToCol(cC));
	}
	
	
	void OnConnectedToServer(){
		connected = true;
		Debug.Log("Connected to a server");
		
		netviewID = Network.AllocateViewID();
		InstantiateFPSEntity(true, netviewID, colA, colB, colC);
		networkView.RPC("NewPlayer", RPCMode.OthersBuffered, netviewID, playerName, ColToVec(colA), ColToVec(colB), ColToVec(colC));
		
	}
	
	void OnServerInitialized() {
        Debug.Log("Server initialized, now registering...");
		MasterServer.RegisterHost("SophiesTestGame", gameName);
    }
	
	
    void OnMasterServerEvent(MasterServerEvent msEvent) {
        if (msEvent == MasterServerEvent.RegistrationSucceeded){
			//success registering
			
			if (!connected){
				netviewID = Network.AllocateViewID();
				InstantiateFPSEntity(true, netviewID, colA, colB, colC);
				networkView.RPC("NewPlayer", RPCMode.OthersBuffered, netviewID, playerName, ColToVec(colA), ColToVec(colB), ColToVec(colC));
			}
			
			isServer = true;
			connected = true;
            Debug.Log("Server registered");
			
		}else if (msEvent == MasterServerEvent.RegistrationFailedNoServer || msEvent == MasterServerEvent.RegistrationFailedGameType || msEvent == MasterServerEvent.RegistrationFailedNoServer){
			//failure registering
			Debug.Log("server registration failed, disconnecting");
			Network.Disconnect();
			
		}
    }
	
	Vector3 ColToVec(Color colIn){
		Vector3 retVec = Vector3.zero;
		retVec.x = colIn.r;
		retVec.y = colIn.g;
		retVec.z = colIn.b;
		return retVec;
	}
	Color VecToCol(Vector3 vecIn){
		Color retCol = Color.white;
		retCol.r = vecIn.x;
		retCol.g = vecIn.y;
		retCol.b = vecIn.z;
		return retCol;
	}
	
	
	void OnPlayerConnected(NetworkPlayer player) {
        Debug.Log("Player connected from " + player.ipAddress + ":" + player.port);
    }
	
	void OnApplicationQuit(){
		if (connected){
			if (!isServer){
				networkView.RPC("PlayerLeave", RPCMode.OthersBuffered, netviewID);
			}else{
				networkView.RPC("ServerLeave", RPCMode.OthersBuffered);
			}
			Network.Disconnect();
			if (isServer) MasterServer.UnregisterHost();
		}
	}
	
	[RPC]
	void PlayerLeave(NetworkViewID viewID){
		Debug.Log("A player did one");
		for (int i=0; i<fpsEntities.Count; i++){
			if (fpsEntities[i].viewID == viewID){
				Destroy(fpsEntities[i].gameObject);
				fpsEntities.RemoveAt(i);
			}
		}
	}
	
	[RPC]
	void ServerLeave(){
		Debug.Log("THE SERVER BUGGERED OFF! HOW RUDE D:");
		
		Network.Disconnect();
		if (isServer) MasterServer.UnregisterHost();
		connected = false;
		isServer = false;
				
		Camera.main.transform.parent = null;
		for (int i=0; i<fpsEntities.Count; i++){
			Destroy(fpsEntities[i].gameObject);
		}
		fpsEntities = new List<FPSEntity>();
	}
	
	
	// Use this for initialization
	void Start () {
	
	}
	
	// Update is called once per frame
	void Update () {
		
	}
}
